#include <windows.h>
#include <string.h>
#include "tty.h"

#define MAXPORTS 258

OVERLAPPED WRITE_OS ;
OVERLAPPED READ_OS ;
HANDLE COMDEV[MAXPORTS] ;
DWORD dwDTR,dwRTS ;
int PORT[MAXPORTS] = { OFF } ;
int t_cflag ;

BOOL APIENTRY DllMAin( HANDLE hInstance, DWORD dwReason, LPVOID lpReserved )
{
    return TRUE ;
}    

//--------------------------------------------------------------------------
//  FUNCTION:IOP_Open(int port, int cflag) 
//
//  PORPOSE:Open a specified serial port, and initialize some 
//          of its parameters.
//--------------------------------------------------------------------------            
int APIENTRY IOP_Open(int port, int cflag) 
{
    COMMTIMEOUTS CommTimeOuts ;
    char szPort[15] ;
    int  bBaud,bSize,bStopBits,bParity; 
    DCB  dcb ;

    if (port < 10)
       wsprintf( szPort, "COM%d", port ) ;  
    else
       wsprintf( szPort, "//./COM%d", port ) ;  
    
    if ((COMDEV[port] =
       CreateFile( szPort,
             GENERIC_READ | GENERIC_WRITE,
             0,      //exclusive access
             NULL,   //no security attrs
             OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL |
             FILE_FLAG_OVERLAPPED,   //overlapped I/O
             NULL )) == (HANDLE)-1)

    return (FAIL) ;
    else
    {
       t_cflag = cflag ;
       
       PORT[port] = ON ;  
       
       // get any early notifications
       SetCommMask( COMDEV[port], EV_RXCHAR ) ;
       
       // setup device buffers
       SetupComm( COMDEV[port], 1024, 1024 ) ;

       // purge any information in the buffer
       PurgeComm( COMDEV[port], PURGE_TXABORT | PURGE_RXABORT |
                             PURGE_TXCLEAR | PURGE_RXCLEAR ) ;
       
       // set up for overlapped I/O
       CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF ;
       CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ;
       CommTimeOuts.ReadTotalTimeoutConstant = 1000 ;
       CommTimeOuts.WriteTotalTimeoutMultiplier = 2 ;
       CommTimeOuts.WriteTotalTimeoutConstant = 0 ;
       
       SetCommTimeouts( COMDEV[port], &CommTimeOuts ) ;
       GetCommState( COMDEV[port], &dcb ) ;    
       
       dcb.DCBlength = sizeof( DCB ) ;

       GetCommState( COMDEV[port], &dcb ) ;
       
       bBaud = cflag & CBAUD ;
       dcb.BaudRate = BaudGroup[bBaud] ;	
       
       bSize = (cflag & CSIZE) >> 4 ;
       dcb.ByteSize = LOBYTE(ByteSizeGroup[bSize]) ;
       
       bStopBits = (cflag & CSTOPB ) >> 6 ;
       dcb.StopBits = LOBYTE(StopBitsGroup[bStopBits]) ;
    
       bParity = (cflag & CPARITYB) >> 8 ;
       dcb.Parity = LOBYTE(ParityGroup[bParity]) ;
       
       // setup hardware flow control
       if (!((cflag & CLOCAL) >> 12))
       {  
          dcb.fDtrControl = DTR_CONTROL_HANDSHAKE ; 
          dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;      
          dcb.fOutxDsrFlow = TRUE ;
          dcb.fOutxCtsFlow = TRUE ;
       }
       else
       {
          dcb.fDtrControl = DTR_CONTROL_ENABLE ; 
          dcb.fRtsControl = RTS_CONTROL_ENABLE ; 
          dcb.fOutxDsrFlow = FALSE ;
          dcb.fOutxCtsFlow = FALSE ;
       }   
       
       // setup software flow control
       if (!((cflag & CREAD) >> 7))
          dcb.fInX = dcb.fOutX = FALSE ;
       else   
          dcb.fInX = dcb.fOutX = TRUE ;
       
       dcb.XonChar = ASCII_XON ;
       dcb.XoffChar = ASCII_XOFF ;
       dcb.XonLim = 100 ;
       dcb.XoffLim = 100 ;
       
       // other various settings
       dcb.fBinary = TRUE ;
       dcb.fParity = TRUE ; 
       
       if (!SetCommState( COMDEV[port], &dcb )) 
       {
          PORT[port] = OFF ;
          return (FAIL) ;
       }
       
       WRITE_OS.Offset = 0 ;
       WRITE_OS.OffsetHigh = 0 ;
       READ_OS.Offset = 0 ;
       READ_OS.OffsetHigh = 0 ;
    
       WRITE_OS.hEvent = CreateEvent( NULL,    // no security
                                     TRUE,    // explicit reset req
                                     FALSE,   // initial event reset
                                     NULL ) ; // no name
    
       READ_OS.hEvent = CreateEvent( NULL,    // no security
                                     TRUE,    // explicit reset req
                                     FALSE,   // initial event reset
                                     NULL ) ; // no name
    }   
    return (OK) ; 
}                    

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Close(int port) 
//
//  PORPOSE:Close a specified serial port.
//--------------------------------------------------------------------------            
int APIENTRY IOP_Close(int port) 
{
    BOOL fRetVal ;
    
    // purge any information in the buffer
    if ((t_cflag & HUPCL) >> 11)
       PurgeComm( COMDEV[port], PURGE_TXABORT | PURGE_RXABORT ) ;
    
    fRetVal = CloseHandle( COMDEV[port] ) ;
    if (!fRetVal)
       return (FAIL) ; 
       
    PORT[port] = OFF ;
    
    return (OK) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Putc(int port, LPSTR c) 
//
//  PROPOSE:Write a character to a specified serial port.
//--------------------------------------------------------------------------            
int APIENTRY IOP_Putc(int port, LPSTR c) 
{
    DWORD dwBytesWritten ;
    BOOL  fRetVal ;
    
    fRetVal = WriteFile( COMDEV[port], c, 1,
                      &dwBytesWritten, &READ_OS ) ;                           
    if (!fRetVal)
       return (FAIL) ;                  

    return (OK) ;
}  

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Getc(int port, LPSTR c) 
//
//  PORPOSE:Read a character from a specified serial port.
//--------------------------------------------------------------------------            
int APIENTRY IOP_Getc(int port, LPSTR c)         
{
    DWORD dwBytesRead ;
    BOOL  fRetVal ;
    
    memset( c, 0x0, 1 ) ;
    
    fRetVal = ReadFile( COMDEV[port], c, 1,
                        &dwBytesRead, &READ_OS ) ;                           
    if (!fRetVal)
       return (FAIL) ;
    
    return (OK) ;
}  

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Write(int port, LPSTR buffer, int count) 
//
//  PORPOSE:Write characters in buffer to a specified serial port.
//--------------------------------------------------------------------------            
DWORD APIENTRY IOP_Write(int port, LPSTR buffer, int count)    
{
    BOOL  fWriteStat ;
    DWORD dwBytesWritten ;
    DWORD dwError ;

    fWriteStat = WriteFile( COMDEV[port], buffer, count,
                           &dwBytesWritten, &WRITE_OS ) ;

    // Note that normally the code will not execute the following
    // because the driver caches write operations. Small I/O requests
    // (up to several thousand bytes) will normally be accepted
    // immediately and WriteFile will return true even though an
    // overlapped operation was specified

    if (!fWriteStat)
    {
       if (GetLastError() == ERROR_IO_PENDING)
       {
          // We should wait for the completion of the write operation
          while(!GetOverlappedResult( COMDEV[port],
                       &WRITE_OS, &dwBytesWritten, TRUE ))
          {
             dwError = GetLastError() ;
             if (dwError == ERROR_IO_INCOMPLETE)
                return (dwBytesWritten) ;
             else
             	return (FAIL) ;   
          }   
       }
       else
       	  return (FAIL) ;	
    }   
    
    return (dwBytesWritten) ;
}  

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Read(int port, LPSTR buffer, int count) 
//
//  PORPOSE:Read characters from a specified serial port.
//--------------------------------------------------------------------------            
DWORD APIENTRY IOP_Read(int port, LPSTR buffer, int count) 
{
    BOOL  fReadStat ;
    DWORD dwError;
    DWORD dwBytesRead ;
    
    memset( buffer, 0x0, 256 ) ;       

    fReadStat = ReadFile( COMDEV[port], buffer, count,
		                   &dwBytesRead, &READ_OS ) ;
    if (!fReadStat)
    {
       if (GetLastError() == ERROR_IO_PENDING)
       {
          // We have to wait for read to complete
          while(!GetOverlappedResult( COMDEV[port],
                       &READ_OS, &dwBytesRead, TRUE ))
          {
             dwError = GetLastError() ;
             if (dwError == ERROR_IO_INCOMPLETE)
                return (dwBytesRead) ;
             else
             	return (FAIL) ;     
          }
       }
       else
          return (FAIL) ;
    }      
    
    return (dwBytesRead) ;
}  

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_DOQ_Count(int port) 
//
//  PORPOSE:Get the number of character in output queue.
//--------------------------------------------------------------------------            
DWORD APIENTRY IOP_DOQ_Count(int port) 
{
    BOOL  fRetVal ;
    DWORD dwError ;
    DWORD dwLength ;
    COMSTAT comstat ;
       
    fRetVal = ClearCommError( COMDEV[port], &dwError, &comstat ) ;      
    
    if (!fRetVal)
       return (FAIL) ; 
    dwLength = comstat.cbOutQue ;
    
    return (dwLength) ;
}
        
//--------------------------------------------------------------------------            
//  FUNCTION:IOP_DIQ_Count(int port) 
//
//  PORPOSE:Get the number of characters in input queue.
//--------------------------------------------------------------------------            
DWORD APIENTRY IOP_DIQ_Count(int port) 
{
    COMSTAT comstat ;
    DWORD dwError ;
    DWORD dwLength ;
    BOOL  fRetVal ;
       
    fRetVal = ClearCommError( COMDEV[port], &dwError, &comstat ) ;      
   
    if (!fRetVal)
       return (FAIL) ;
    dwLength = comstat.cbInQue ;
    
    return (dwLength) ;
}
    

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_ResetDOQ(int port) 
//
//  PORPOSE:Flush all the data int the output queue of a specified port.
//--------------------------------------------------------------------------            
int APIENTRY IOP_ResetDOQ(int port)
{
    BOOL fRetVal ;
    
    fRetVal = PurgeComm( COMDEV[port], PURGE_TXCLEAR ) ;
    
    if (!fRetVal)
       return (FAIL) ;

    return (OK) ;
}
    
//--------------------------------------------------------------------------            
//  FUNCTION:IOP_ResetDIQ(int port) 
//
//  PORPOSE:Clear all the data int the input queue of a specified port.
//--------------------------------------------------------------------------            
int APIENTRY IOP_ResetDIQ(int port)
{
    BOOL fRetVal ;

    fRetVal = PurgeComm( COMDEV[port], PURGE_RXCLEAR ) ;

    if (!fRetVal)
       return (FAIL) ;
    
    return (TRUE) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_SetDTR(int port, int flag) 
//
//  PORPOSE:Set the DTR state of a specified serial port ON or OFF.
//--------------------------------------------------------------------------
int APIENTRY IOP_SetDTR(int port, int flag) 
{ 
    BOOL  fRetVal ;
    DCB   dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;    
    
    if (dcb.fDtrControl != DTR_CONTROL_DISABLE)
       dwDTR = dcb.fDtrControl ;
    
    if (flag == OFF)
       dcb.fDtrControl = DTR_CONTROL_DISABLE ;  

    if (flag == ON)
       dcb.fDtrControl = dwDTR ;
    
    fRetVal = SetCommState( COMDEV[port], &dcb ) ;    
    
    if (!fRetVal)
       return (FAIL) ;
        
    return (OK) ;
}            

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_SetRTS(int port, int flag) 
//
//  PORPOSE:Set the RTS state of a specified serial port ON or OFF.
//--------------------------------------------------------------------------
int APIENTRY IOP_SetRTS(int port, int flag) 
{ 
    BOOL  fRetVal ;
    DCB   dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;    
    
    if (dcb.fRtsControl != RTS_CONTROL_DISABLE)
       dwRTS = dcb.fRtsControl ;
    
    if (flag == OFF)
       dcb.fRtsControl = RTS_CONTROL_DISABLE ;  

    if (flag == ON)
       dcb.fRtsControl = dwRTS ;
    
    fRetVal = SetCommState( COMDEV[port], &dcb ) ;    
        
    if (!fRetVal)
       return (FAIL) ;    
    
    return (OK) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Ioctrl(int port, int cflag) 
//
//  PORPOSE:Change the parameter of a specific serial port.  
//--------------------------------------------------------------------------
int APIENTRY IOP_Ioctrl(int port, int cflag) 
{ 
    BOOL fRetVal ;
    int  bBaud,bSize,bStopBits,bParity ; 
    DCB  dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;    
    
    t_cflag = cflag ;
    
    bBaud = cflag & CBAUD ;
    dcb.BaudRate = BaudGroup[bBaud] ;	
    
    bSize = (cflag & CSIZE) >> 4 ;
    dcb.ByteSize = LOBYTE(ByteSizeGroup[bSize]) ;
    
    bStopBits = (cflag & CSTOPB ) >> 6 ;
    dcb.StopBits = LOBYTE(StopBitsGroup[bStopBits]) ;
    
    bParity = (cflag & CPARITYB) >> 8 ;
    dcb.Parity = LOBYTE(ParityGroup[bParity]) ;
    
    // setup hardware flow control
    if (!((cflag & CLOCAL) >> 12))
    {  
       dcb.fDtrControl = DTR_CONTROL_HANDSHAKE ; 
       dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;      
       dcb.fOutxDsrFlow = TRUE ;
       dcb.fOutxCtsFlow = TRUE ;
    }
    else
    {
       dcb.fDtrControl = DTR_CONTROL_ENABLE ; 
       dcb.fRtsControl = RTS_CONTROL_ENABLE ; 
       dcb.fOutxDsrFlow = FALSE ;
       dcb.fOutxCtsFlow = FALSE ;
    }   
    
    // setup software flow control
    if (!((cflag & CREAD) >> 7))
       dcb.fInX = dcb.fOutX = FALSE ;
    else   
       dcb.fInX = dcb.fOutX = TRUE ;
    
    fRetVal = SetCommState( COMDEV[port], &dcb ) ;
 
    if (!fRetVal)
       return (FAIL) ;
 
    return (OK) ;      
}       

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Flowctrl(int port, int cflag) 
//
//  PORPOSE:Set hardware and software flow control.
//--------------------------------------------------------------------------
int APIENTRY IOP_Flowctrl(int port, int cflag) 
{ 
    BOOL fRetVal ;
    BYTE bSet ;
    DCB  dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;    
    
    // setup hardware flow control
    bSet = (BYTE)((cflag & FC_DTRDSR) != 0) ;
    dcb.fOutxDsrFlow = bSet ;
    if (bSet)
       dcb.fDtrControl = DTR_CONTROL_HANDSHAKE ;  
    else
       dcb.fDtrControl = DTR_CONTROL_ENABLE ;
    
    bSet = (BYTE)((cflag & FC_RTSCTS) != 0) ;
    dcb.fOutxCtsFlow = bSet ;
    if (bSet)
       dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;  
    else
       dcb.fRtsControl = RTS_CONTROL_ENABLE ;     
    
    // setup software flow control   
    bSet = (BYTE)((cflag & FC_XONXOFF) != 0) ;
    dcb.fInX = dcb.fOutX = bSet ;
    
    fRetVal = SetCommState( COMDEV[port], &dcb ) ;    
    if (!fRetVal)
       return (FAIL) ;
        
    return (OK) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_GetIoctrl(int port) 
//
//  PORPOSE:Get the parameter of a specific serial port.  .
//--------------------------------------------------------------------------            
int APIENTRY IOP_GetIoctrl(int port)
{
    int bBaud,bSize,bStopBits,bParity; 
    int bDTRRTS,bXONXOFF,bHupcl,i,cflag ; 
    DCB dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;          
    
    for( i=0; i<15; i++ )
    {
       if (dcb.BaudRate == BaudGroup[i])
          bBaud = i ;
    }                                    
    
    for( i=0; i<4; i++ )
    {      
       if (dcb.ByteSize == ByteSizeGroup[i])
          bSize = i << 4 ;
    }      
    
    for( i=0; i<2; i++ )
    {      
       if (dcb.StopBits == StopBitsGroup[i])
          bStopBits = i << 6 ;
    }
                
    for( i=0; i<5; i++ )
    {      
       if (dcb.Parity == ParityGroup[i])
          bParity = i << 8 ;
    }            
    
    if ((dcb.fDtrControl == DTR_CONTROL_ENABLE) &&
                     (dcb.fRtsControl == RTS_CONTROL_ENABLE))  
       bDTRRTS = CLOCAL ;
    else
       bDTRRTS = 0 ;
 
    if (dcb.fInX != 0)  
       bXONXOFF = CREAD ;
    else   
       bXONXOFF = 0 ;
          
    bHupcl = t_cflag & HUPCL ;
       
    cflag = bBaud | bSize | bStopBits | bParity |
                                bDTRRTS | bXONXOFF | bHupcl;
    
    return (cflag) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_GetFlowctrl(int port)
//
//  PORPOSE:Get hardware and software flow control.
//--------------------------------------------------------------------------
int APIENTRY IOP_GetFlowctrl(int port) 
{ 
    int  bDTRDSR,bRTSCTS,bXONXOFF ; 
    int  iflag ;
    DCB  dcb ;
    
    GetCommState( COMDEV[port], &dcb ) ;    
    
    if (dcb.fDtrControl == DTR_CONTROL_HANDSHAKE) 
       bDTRDSR = FC_DTRDSR ;
    else
    {
       if (dcb.fDtrControl == DTR_CONTROL_ENABLE) 
       bDTRDSR = 0 ;
       if (dcb.fDtrControl == DTR_CONTROL_DISABLE)
       bDTRDSR = 0x08 ; 
    }   
    
    if (dcb.fRtsControl == RTS_CONTROL_HANDSHAKE) 
       bRTSCTS = FC_RTSCTS ;
    else   
    {
       if (dcb.fRtsControl == RTS_CONTROL_ENABLE) 
       bRTSCTS = 0 ;
       if (dcb.fRtsControl == RTS_CONTROL_DISABLE) 
       bRTSCTS = 0x10 ;
    }   
       
    if (dcb.fInX != 0)  
       bXONXOFF = FC_XONXOFF ;
    else   
       bXONXOFF = 0 ;
       
    iflag = bDTRDSR | bRTSCTS | bXONXOFF ;
    
    return (iflag) ;
}

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_Status(int port)
//
//  PORPOSE:Get the status of a specific serial port.
//--------------------------------------------------------------------------     
int APIENTRY IOP_Status(int port) 
{
    DCB dcb ;
    int sflag,i ;
    int bBaud,bOpen,bModem,bCTS,bRX,bDCD,bTX,bDSR ;
    DWORD dwEvent,dwStatus ;
    
    dwEvent = dwStatus = 0 ;
    GetCommState( COMDEV[port], &dcb ) ;          
    WaitCommEvent( COMDEV[port], &dwEvent, &WRITE_OS ) ;
    GetCommModemStatus( COMDEV[port], &dwStatus ) ;
    
    for( i=0; i<15; i++ )
    {
       if (dcb.BaudRate == BaudGroup[i])
          bBaud = i & CBAUD ;
       if (PORT[port] == OFF ) 
          bBaud = 0x0000 ;   
    }                                    
    
    if (PORT[port] == ON)
       bOpen = S_OPEN ;
    else
       bOpen = 0x0000 ;
    
    if ((dcb.fDtrControl == DTR_CONTROL_HANDSHAKE) && 
          (dcb.fRtsControl == RTS_CONTROL_HANDSHAKE)) 
       bModem = S_MODEM ;
    else
       bModem = 0x0000 ;             
       
    if ((dwStatus & MS_CTS_ON) == MS_CTS_ON)
       bCTS = S_CTS ;
    else   
       bCTS = 0x0000 ; 
       
    if ((dwEvent & EV_RXCHAR) == EV_RXCHAR) 
       bRX = S_RX ;
    else   
       bRX = 0x0000 ; 
    
    if ((dwStatus & MS_RLSD_ON) == MS_RLSD_ON) 
       bDCD = S_DCD ;
    else
       bDCD = 0x0000 ;   

    if (PORT[port] == OFF)
       bTX = S_TX ;
    else
       bTX = 0x0000 ;
    
    if ((dwStatus & MS_DSR_ON) == MS_DSR_ON)
       bDSR = S_DSR ;
    else
       bDSR = 0x0000 ;
          
    sflag = bBaud | bOpen | bModem | bCTS |
                         bRX | bDCD | bTX | bDSR ;
                                        
    return ( sflag ) ;

}        

//--------------------------------------------------------------------------            
//  FUNCTION:IOP_DataStatus(int port)
//
//  PORPOSE:Chech if any error happened when receiving data.
//--------------------------------------------------------------------------     
int APIENTRY IOP_DataStatus(int port) 
{
    int bOverrun,bOverflow,bParityError,bTxFull ;
    int eflag ;
    DWORD dwError ;
    COMSTAT comstat ;
    
    dwError = 0 ;
    ClearCommError( COMDEV[port], &dwError, &comstat ) ;
    
    if ((dwError & CE_OVERRUN) == CE_OVERRUN)
       bOverrun = E_OVERRUN ;
    else 
       bOverrun = 0x00 ;
          
    if ((dwError & CE_RXOVER) == CE_RXOVER)
       bOverflow = E_OVERFLOW ;
    else
       bOverflow = 0x00 ;
       
    if ((dwError & CE_RXPARITY) == CE_RXPARITY)
       bParityError = E_PARITY ;
    else
       bParityError = 0x00 ;
    
    if ((dwError & CE_TXFULL) == CE_TXFULL)
       bTxFull = E_TXFULL ;
    else
       bTxFull = 0x00 ;
       
    eflag = bOverrun | bOverflow | bParityError | bTxFull ;
    
    return (eflag) ;

}
       
//---------------------------------------------------------------------------
//  Enf of File: tty.c
//---------------------------------------------------------------------------